# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2018-09-11 15:29
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nokiaapp', '0003_rename_nokiaapp'),
    ]

    operations = [
        migrations.AddField(
            model_name='nokiauser',
            name='refresh_token',
            field=models.TextField(default='', help_text=b'OAuth2 refresh token'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='nokiauser',
            name='token_expiry',
            field=models.IntegerField(default=0, help_text=b'Token expiration timestamp'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='nokiauser',
            name='token_type',
            field=models.CharField(default='', help_text=b'Type of OAuth2 token', max_length=32),
            preserve_default=False,
        ),
    ]
